@@ -10,6 +10,7 @@ Copyright (c) 2009 David Peckham. All rights reserved.
 import os
 import re
 import sys
+from repr import repr
 
 
 class Universe(object):
@@ -27,6 +28,13 @@ class Universe(object):
         self.joins = []
         self.parameters = None
 
+    def table_name(self, table_id):
+        try:
+            tablename = self.table_map[table_id].name
+        except KeyError:
+            tablename = 'UNKNOWN TABLE (%d)' % table_id
+        return tablename
+
 
 class Parameters(object):
     """docstring for Parameters"""
@@ -66,7 +74,34 @@ class Class(object):
         self.subclasses = []
 
 
+class Join(object):
+    
+    """docstring for Join"""
+    def __init__(self, universe):
+        super(Join, self).__init__()
+        self.universe = universe
+        self.expression = None
+        self.term_count = 0
+        self.terms = []
+    
+    @property
+    def statement(self):
+        #TODO: add table or object names to the statement
+        if self.term_count == 2:
+            s = self.fullterm(self.terms[0]) + self.expression + self.fullterm(self.terms[1])
+        else:
+            format = self.expression.replace(chr(1), '%s')
+            s = format % tuple([self.fullterm(t) for t in self.terms])
+        return s
+    
+    def fullterm(self, term):
+        """return the fully qualified term with table and column names"""
+        column_name, table_id = term
+        return '%s.%s' % (self.universe.table_name(table_id), column_name)
+
+
 class ObjectBase(object):
+    
     """docstring for ObjectBase"""
     def __init__(self, universe, id_, parent, name, description):
         super(ObjectBase, self).__init__()
@@ -165,12 +200,12 @@ class Column(object):
 
     @property
     def table_name(self):
-        try:
-            tablename = self.universe.table_map[self.parent].name
-        except KeyError:
-            tablename = 'UNKNOWN TABLE (%d)' % self.parent
-        return tablename
+        return self.universe.table_name(self.parent)
 
+        
+    def __cmp__(self, other):
+        return cmp(self.id_, other.id_)
+        
     def __str__(self):
         return '%s id=%d, name=%s parent=%d' % (type(self), 
             self.id_, self.name, self.parent) 
